@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2008-2010 by Andrzej Rybczak                            *
+ *   Copyright (C) 2008-2012 by Andrzej Rybczak                            *
  *   electricityispower@gmail.com                                          *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -34,7 +34,7 @@ namespace Display
 		std::string *format;
 	};
 	
-	std::string Columns();
+	std::string Columns(size_t);
 	
 	template <typename T> void Generic(const T &t, void *, Menu<T> *menu)
 	{
