@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2008-2010 by Andrzej Rybczak                            *
+ *   Copyright (C) 2008-2012 by Andrzej Rybczak                            *
  *   electricityispower@gmail.com                                          *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -38,8 +38,6 @@
 
 using Global::MainHeight;
 using Global::MainStartY;
-using Global::myScreen;
-using Global::myOldScreen;
 
 Lastfm *myLastfm = new Lastfm;
 
@@ -51,8 +49,10 @@ void Lastfm::Init()
 
 void Lastfm::Resize()
 {
-	w->Resize(COLS, MainHeight);
-	w->MoveTo(0, MainStartY);
+	size_t x_offset, width;
+	GetWindowResizeParams(x_offset, width);
+	w->Resize(width, MainHeight);
+	w->MoveTo(x_offset, MainStartY);
 	hasToBeResized = 0;
 }
 
@@ -79,13 +79,20 @@ void Lastfm::Take()
 
 void Lastfm::SwitchTo()
 {
+	using Global::myScreen;
+	using Global::myOldScreen;
+	using Global::myLockedScreen;
+	
 	if (myScreen == this)
 		return myOldScreen->SwitchTo();
 	
 	if (!isInitialized)
 		Init();
 	
-	if (hasToBeResized)
+	if (myLockedScreen)
+		UpdateInactiveScreen(this);
+	
+	if (hasToBeResized || myLockedScreen)
 		Resize();
 	
 	// get an old info if it waits
@@ -159,7 +166,7 @@ void Lastfm::SetTitleAndFolder()
 	{
 		itsTitle = U("Artist info - ");
 		itsTitle += TO_WSTRING(itsArgs.find("artist")->second);
-		itsFolder = home_path + HOME_FOLDER + "artists";
+		itsFolder = Config.ncmpcpp_directory + "artists";
 	}
 }
 
