@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2008-2010 by Andrzej Rybczak                            *
+ *   Copyright (C) 2008-2012 by Andrzej Rybczak                            *
  *   electricityispower@gmail.com                                          *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -22,6 +22,7 @@
 #define _LYRICS_H
 
 #include <pthread.h>
+#include <queue>
 
 #include "ncmpcpp.h"
 #include "mpdpp.h"
@@ -33,7 +34,7 @@ class Lyrics : public Screen<Scrollpad>
 	public:
 		Lyrics() : ReloadNP(0),
 #		ifdef HAVE_CURL_CURL_H
-		isReadyToTake(0), isDownloadInProgress(0), itsFetcher(0),
+		isReadyToTake(0), isDownloadInProgress(0),
 #		endif // HAVE_CURL_CURL_H
 		itsScrollBegin(0) { }
 		
@@ -51,39 +52,55 @@ class Lyrics : public Screen<Scrollpad>
 		
 		virtual List *GetList() { return 0; }
 		
+		virtual bool isMergable() { return true; }
+		
 		void Edit();
 		void Refetch();
+		
 #		ifdef HAVE_CURL_CURL_H
-		void ToggleFetcher();
+		static void ToggleFetcher();
+		static void DownloadInBackground(const MPD::Song *s);
 #		endif // HAVE_CURL_CURL_H
 		
 		bool ReloadNP;
 		
 	protected:
 		virtual void Init();
+		virtual bool isLockable() { return false; }
 		
 	private:
 		void Load();
 		
-		void SetFilename();
-		std::string itsFilename;
-		std::string itsFolder;
-		
 #		ifdef HAVE_CURL_CURL_H
+		static void *DownloadInBackgroundImpl(void *);
+		static void DownloadInBackgroundImplHelper(MPD::Song *);
+		// lock for allowing exclusive access to itsToDownload and itsWorkersNumber
+		static pthread_mutex_t itsDIBLock;
+		// storage for songs for which lyrics are scheduled to be downloaded
+		static std::queue<MPD::Song *> itsToDownload;
+		// current worker threads (ie. downloading lyrics)
+		static size_t itsWorkersNumber;
+		// maximum number of worker threads. if it's reached, next lyrics requests
+		// are put into itsToDownload queue.
+		static const size_t itsMaxWorkersNumber = 4;
+		
 		void *Download();
 		static void *DownloadWrapper(void *);
-		
-		void Save(const std::string &lyrics);
+		static void Save(const std::string &filename, const std::string &lyrics);
 		
 		void Take();
 		bool isReadyToTake;
 		bool isDownloadInProgress;
 		pthread_t itsDownloader;
-		LyricsFetcher **itsFetcher;
+		
+		static LyricsFetcher **itsFetcher;
 #		endif // HAVE_CURL_CURL_H
 		
 		size_t itsScrollBegin;
 		MPD::Song itsSong;
+		std::string itsFilename;
+		
+		static std::string GenerateFilename(const MPD::Song &s);
 };
 
 extern Lyrics *myLyrics;
