@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2008-2010 by Andrzej Rybczak                            *
+ *   Copyright (C) 2008-2012 by Andrzej Rybczak                            *
  *   electricityispower@gmail.com                                          *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -682,6 +682,24 @@ void MPD::Connection::GetPlaylistContent(const std::string &path, SongList &v)
 	GoIdle();
 }
 
+void MPD::Connection::GetSupportedExtensions(std::set<std::string> &acc)
+{
+	if (!itsConnection)
+		return;
+	assert(!isCommandsListEnabled);
+	GoBusy();
+	
+	mpd_send_command(itsConnection, "decoders", NULL);
+	while (mpd_pair *pair = mpd_recv_pair_named(itsConnection, "suffix"))
+	{
+		acc.insert(pair->value);
+		mpd_return_pair(itsConnection, pair);
+	}
+	mpd_response_finish(itsConnection);
+	
+	GoIdle();
+}
+
 void MPD::Connection::SetRepeat(bool mode)
 {
 	if (!itsConnection)
@@ -933,7 +951,7 @@ bool MPD::Connection::AddRandomSongs(size_t number)
 		srand(time(0));
 		std::random_shuffle(files.begin(), files.end());
 		StartCommandsList();
-		TagList::const_iterator it = files.begin()+rand()%(files.size()-number);
+		TagList::const_iterator it = files.begin()+rand()%(std::max(size_t(1), files.size()-number));
 		for (size_t i = 0; i < number && it != files.end(); ++i)
 			AddSong(*it++);
 		CommitCommandsList();
