@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2008-2010 by Andrzej Rybczak                            *
+ *   Copyright (C) 2008-2012 by Andrzej Rybczak                            *
  *   electricityispower@gmail.com                                          *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -21,6 +21,7 @@
 #ifndef _MPDPP_H
 #define _MPDPP_H
 
+#include <set>
 #include <vector>
 
 #include <mpd/client.h>
@@ -162,6 +163,8 @@ namespace MPD
 			Song GetSong(const std::string &);
 			void GetPlaylistContent(const std::string &, SongList &);
 			
+			void GetSupportedExtensions(std::set<std::string> &);
+			
 			void SetRepeat(bool);
 			void SetRandom(bool);
 			void SetSingle(bool);
