@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2008-2010 by Andrzej Rybczak                            *
+ *   Copyright (C) 2008-2012 by Andrzej Rybczak                            *
  *   electricityispower@gmail.com                                          *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -75,6 +75,8 @@ using namespace MPD;
 BasicScreen *Global::myScreen;
 BasicScreen *Global::myOldScreen;
 BasicScreen *Global::myPrevScreen;
+BasicScreen *Global::myLockedScreen;
+BasicScreen *Global::myInactiveScreen;
 
 Window *Global::wHeader;
 Window *Global::wFooter;
@@ -107,6 +109,40 @@ namespace
 		}
 	}
 	
+	void set_resize_flags()
+	{
+		myHelp->hasToBeResized = 1;
+		myPlaylist->hasToBeResized = 1;
+		myBrowser->hasToBeResized = 1;
+		mySearcher->hasToBeResized = 1;
+		myLibrary->hasToBeResized = 1;
+		myPlaylistEditor->hasToBeResized = 1;
+		myLyrics->hasToBeResized = 1;
+		mySelectedItemsAdder->hasToBeResized = 1;
+		mySongInfo->hasToBeResized = 1;
+		
+#		ifdef HAVE_CURL_CURL_H
+		myLastfm->hasToBeResized = 1;
+#		endif // HAVE_CURL_CURL_H
+		
+#		ifdef HAVE_TAGLIB_H
+		myTinyTagEditor->hasToBeResized = 1;
+		myTagEditor->hasToBeResized = 1;
+#		endif // HAVE_TAGLIB_H
+		
+#		ifdef ENABLE_VISUALIZER
+		myVisualizer->hasToBeResized = 1;
+#		endif // ENABLE_VISUALIZER
+		
+#		ifdef ENABLE_OUTPUTS
+		myOutputs->hasToBeResized = 1;
+#		endif // ENABLE_OUTPUTS
+		
+#		ifdef ENABLE_CLOCK
+		myClock->hasToBeResized = 1;
+#		endif // ENABLE_CLOCK
+	}
+	
 	void resize_screen()
 	{
 		order_resize = 0;
@@ -139,38 +175,9 @@ namespace
 		if (!Config.statusbar_visibility)
 			MainHeight++;
 		
-		myHelp->hasToBeResized = 1;
-		myPlaylist->hasToBeResized = 1;
-		myBrowser->hasToBeResized = 1;
-		mySearcher->hasToBeResized = 1;
-		myLibrary->hasToBeResized = 1;
-		myPlaylistEditor->hasToBeResized = 1;
-		myLyrics->hasToBeResized = 1;
-		mySelectedItemsAdder->hasToBeResized = 1;
-		mySongInfo->hasToBeResized = 1;
+		set_resize_flags();
 		
-#		ifdef HAVE_CURL_CURL_H
-		myLastfm->hasToBeResized = 1;
-#		endif // HAVE_CURL_CURL_H
-		
-#		ifdef HAVE_TAGLIB_H
-		myTinyTagEditor->hasToBeResized = 1;
-		myTagEditor->hasToBeResized = 1;
-#		endif // HAVE_TAGLIB_H
-		
-#		ifdef ENABLE_VISUALIZER
-		myVisualizer->hasToBeResized = 1;
-#		endif // ENABLE_VISUALIZER
-		
-#		ifdef ENABLE_OUTPUTS
-		myOutputs->hasToBeResized = 1;
-#		endif // ENABLE_OUTPUTS
-		
-#		ifdef ENABLE_CLOCK
-		myClock->hasToBeResized = 1;
-#		endif // ENABLE_CLOCK
-		
-		myScreen->Resize();
+		ApplyToVisibleWindows(&BasicScreen::Resize);
 		
 		if (Config.header_visibility || Config.new_design)
 			wHeader->Resize(COLS, header_height);
@@ -179,7 +186,7 @@ namespace
 		wFooter->MoveTo(0, footer_start_y);
 		wFooter->Resize(COLS, Config.statusbar_visibility ? 2 : 1);
 		
-		myScreen->Refresh();
+		ApplyToVisibleWindows(&BasicScreen::Refresh);
 		RedrawStatusbar = 1;
 		StatusChanges changes;
 		if (!Mpd.isPlaying() || design_changed)
@@ -201,6 +208,7 @@ namespace
 			ShowMessage("User interface: %s", Config.new_design ? "Alternative" : "Classic");
 		}
 		wFooter->Refresh();
+		refresh();
 	}
 	
 #	if !defined(WIN32)
@@ -230,10 +238,11 @@ namespace
 	}
 }
 
-int main(int argc, char *argv[])
+int main(int argc, char **argv)
 {
 	setlocale(LC_ALL, "");
-	CreateConfigDir();
+	
+	Config.CheckForCommandLineConfigFilePath(argv, argc);
 	
 	Config.SetDefaults();
 	Key.SetDefaults();
@@ -257,17 +266,19 @@ int main(int argc, char *argv[])
 		ParseArgv(argc, argv);
 	
 	if (!ConnectToMPD())
-		return -1;
+		exit(1);
+	
+	CreateDir(Config.ncmpcpp_directory);
 	
 	// always execute these commands, even if ncmpcpp use exit function
 	atexit(do_at_exit);
 	
 	// redirect std::cerr output to ~/.ncmpcpp/error.log file
-	errorlog.open((config_dir + "error.log").c_str(), std::ios::app);
+	errorlog.open((Config.ncmpcpp_directory + "error.log").c_str(), std::ios::app);
 	cerr_buffer = std::cerr.rdbuf();
 	std::cerr.rdbuf(errorlog.rdbuf());
 	
-	InitScreen("ncmpc++ ver. "VERSION, Config.colors_enabled);
+	InitScreen("ncmpcpp ver. "VERSION, Config.colors_enabled);
 	
 	bool real_statusbar_visibility = Config.statusbar_visibility;
 	
@@ -323,8 +334,6 @@ int main(int argc, char *argv[])
 	signal(SIGWINCH, sighandler);
 #	endif // !WIN32
 	
-	MEVENT mouse_event, old_mouse_event;
-	old_mouse_event.bstate = 0;
 	mouseinterval(0);
 	if (Config.mouse_support)
 		mousemask(ALL_MOUSE_EVENTS, 0);
@@ -450,9 +459,6 @@ int main(int argc, char *argv[])
 			RedrawHeader = 1;
 		title_allowed = 1;
 		
-		if (myScreen == myPlaylist)
-			myPlaylist->EnableHighlighting();
-		
 		// key mapping beginning
 		
 		if (Keypressed(input, Key.Up))
@@ -526,6 +532,8 @@ int main(int argc, char *argv[])
 		}
 		else if (Config.mouse_support && input == KEY_MOUSE)
 		{
+			static MEVENT mouse_event, old_mouse_event;
+			old_mouse_event = mouse_event;
 			getmouse(&mouse_event);
 			// workaround shitty ncurses behavior introduced in >=5.8, when we mysteriously get
 			// a few times after ncmpcpp startup 2^27 code instead of BUTTON{1,3}_RELEASED. since that
@@ -566,7 +574,6 @@ int main(int argc, char *argv[])
 			}
 			else if (mouse_event.bstate & (BUTTON1_PRESSED | BUTTON2_PRESSED | BUTTON3_PRESSED | BUTTON4_PRESSED))
 				myScreen->MouseButtonPressed(mouse_event);
-			old_mouse_event = mouse_event;
 		}
 		else if (Keypressed(input, Key.ToggleInterface))
 		{
@@ -607,53 +614,27 @@ int main(int argc, char *argv[])
 		{
 			myScreen->SpacePressed();
 		}
-		else if (Keypressed(input, Key.PrevColumn)
-		     &&  (myScreen == myLibrary
-		       || myScreen == myPlaylistEditor
-#		ifdef HAVE_TAGLIB_H
-		       || myScreen == myTagEditor
-#		endif // HAVE_TAGLIB_H)
-			 )
-			)
+		else if (Keypressed(input, Key.PrevColumn) && SwitchToPrevColumn(myScreen)) { }
+		else if (Keypressed(input, Key.NextColumn) && SwitchToNextColumn(myScreen)) { }
+		else if (Keypressed(input, Key.PrevColumn) && myLockedScreen && myInactiveScreen
+		     &&  myLockedScreen != myScreen && myScreen->isMergable())
 		{
-			if (myScreen == myLibrary)
+			if (myScreen != myLockedScreen)
 			{
-				myLibrary->PrevColumn();
+				myInactiveScreen = myScreen;
+				myScreen = myLockedScreen;
+				RedrawHeader = 1;
 			}
-			else if (myScreen == myPlaylistEditor)
-			{
-				myPlaylistEditor->PrevColumn();
-			}
-#			ifdef HAVE_TAGLIB_H
-			else if (myScreen == myTagEditor)
-			{
-				myTagEditor->PrevColumn();
-			}
-#			endif // HAVE_TAGLIB_H
 		}
-		else if (Keypressed(input, Key.NextColumn)
-		     &&  (myScreen == myLibrary
-		       || myScreen == myPlaylistEditor
-#		ifdef HAVE_TAGLIB_H
-		       || myScreen == myTagEditor
-#		endif // HAVE_TAGLIB_H)
-			 )
-			)
+		else if (Keypressed(input, Key.NextColumn) && myLockedScreen && myInactiveScreen
+		     &&  myLockedScreen == myScreen && myScreen->isMergable())
 		{
-			if (myScreen == myLibrary)
+			if (myScreen == myLockedScreen)
 			{
-				myLibrary->NextColumn();
+				myScreen = myInactiveScreen;
+				myInactiveScreen = myLockedScreen;
+				RedrawHeader = 1;
 			}
-			else if (myScreen == myPlaylistEditor)
-			{
-				myPlaylistEditor->NextColumn();
-			}
-#			ifdef HAVE_TAGLIB_H
-			else if (myScreen == myTagEditor)
-			{
-				myTagEditor->NextColumn();
-			}
-#			endif // HAVE_TAGLIB_H
 		}
 		else if (Keypressed(input, Key.VolumeUp))
 		{
@@ -1397,7 +1378,7 @@ int main(int argc, char *argv[])
 				if (Config.columns_in_playlist)
 				{
 					myPlaylist->Items->SetItemDisplayer(Display::SongsInColumns);
-					myPlaylist->Items->SetTitle(Config.titles_visibility ? Display::Columns() : "");
+					myPlaylist->Items->SetTitle(Config.titles_visibility ? Display::Columns(myPlaylist->Items->GetWidth()) : "");
 					myPlaylist->Items->SetGetStringFunction(Playlist::SongInColumnsToString);
 				}
 				else
@@ -1411,7 +1392,7 @@ int main(int argc, char *argv[])
 			{
 				Config.columns_in_browser = !Config.columns_in_browser;
 				ShowMessage("Browser display mode: %s", Config.columns_in_browser ? "Columns" : "Classic");
-				myBrowser->Main()->SetTitle(Config.columns_in_browser && Config.titles_visibility ? Display::Columns() : "");
+				myBrowser->Main()->SetTitle(Config.columns_in_browser && Config.titles_visibility ? Display::Columns(myBrowser->Main()->GetWidth()) : "");
 				
 			}
 			else if (myScreen == mySearcher)
@@ -1419,7 +1400,22 @@ int main(int argc, char *argv[])
 				Config.columns_in_search_engine = !Config.columns_in_search_engine;
 				ShowMessage("Search engine display mode: %s", Config.columns_in_search_engine ? "Columns" : "Classic");
 				if (mySearcher->Main()->Size() > SearchEngine::StaticOptions)
-					mySearcher->Main()->SetTitle(Config.columns_in_search_engine && Config.titles_visibility ? Display::Columns() : "");
+					mySearcher->Main()->SetTitle(Config.columns_in_search_engine && Config.titles_visibility ? Display::Columns(mySearcher->Main()->GetWidth()) : "");
+			}
+			else if (myScreen->ActiveWindow() == myPlaylistEditor->Content)
+			{
+				Config.columns_in_playlist_editor = !Config.columns_in_playlist_editor;
+				ShowMessage("Playlist editor display mode: %s", Config.columns_in_playlist_editor ? "Columns" : "Classic");
+				if (Config.columns_in_playlist_editor)
+				{
+					myPlaylistEditor->Content->SetItemDisplayer(Display::SongsInColumns);
+					myPlaylistEditor->Content->SetGetStringFunction(Playlist::SongInColumnsToString);
+				}
+				else
+				{
+					myPlaylistEditor->Content->SetItemDisplayer(Display::Songs);
+					myPlaylistEditor->Content->SetGetStringFunction(Playlist::SongToString);
+				}
 			}
 		}
 		else if (Keypressed(input, Key.ToggleSeparatorsInPlaylist))
@@ -1432,6 +1428,11 @@ int main(int argc, char *argv[])
 		{
 			myLyrics->ToggleFetcher();
 		}
+		else if (Keypressed(input, Key.ToggleFetchingLyricsInBackground))
+		{
+			Config.fetch_lyrics_in_background = !Config.fetch_lyrics_in_background;
+			ShowMessage("Fetching lyrics for currently playing song in background: %s", Config.fetch_lyrics_in_background ? "On" : "Off");
+		}
 #		endif // HAVE_CURL_CURL_H
 		else if (Keypressed(input, Key.ToggleAutoCenter))
 		{
@@ -1731,6 +1732,39 @@ int main(int argc, char *argv[])
 			if (s)
 				myLibrary->LocateSong(*s);
 		}
+		else if (Keypressed(input, Key.ToggleScreenLock))
+		{
+			if (myLockedScreen != 0)
+			{
+				BasicScreen::Unlock();
+				set_resize_flags();
+				ShowMessage("Screen unlocked");
+			}
+			else
+			{
+				int part = Config.locked_screen_width_part*100;
+				if (Config.ask_for_locked_screen_width_part)
+				{
+					LockStatusbar();
+					Statusbar() << "% of the locked screen's width to be reserved (20-80): ";
+					std::string str_part = wFooter->GetString(IntoStr(Config.locked_screen_width_part*100));
+					UnlockStatusbar();
+					if (str_part.empty())
+						continue;
+					part = StrToInt(str_part);
+				}
+				if (part < 20 || part > 80)
+				{
+					ShowMessage("Invalid number (%d)!", part);
+					continue;
+				}
+				Config.locked_screen_width_part = part/100.0;
+				if (myScreen->Lock())
+					ShowMessage("Screen locked (with %d%% width)", part);
+				else
+					ShowMessage("Screen cannot be locked");
+			}
+		}
 #		ifdef HAVE_TAGLIB_H
 		else if (Keypressed(input, Key.GoToTagEditor))
 		{
@@ -2348,10 +2382,15 @@ int main(int argc, char *argv[])
 			}
 		}
 		
+		if (myScreen == myPlaylist)
+			myPlaylist->EnableHighlighting();
+		
 #		ifdef ENABLE_VISUALIZER
 		// visualizer sets timmeout to 40ms, but since only it needs such small
 		// value, we should restore defalt one after switching to another screen.
-		if (wFooter->GetTimeout() < ncmpcpp_window_timeout && myScreen != myVisualizer)
+		if (wFooter->GetTimeout() < ncmpcpp_window_timeout
+		&&  !(myScreen == myVisualizer || myLockedScreen == myVisualizer || myInactiveScreen == myVisualizer)
+		   )
 			wFooter->SetTimeout(ncmpcpp_window_timeout);
 #		endif // ENABLE_VISUALIZER
 	}
