@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2008-2010 by Andrzej Rybczak                            *
+ *   Copyright (C) 2008-2012 by Andrzej Rybczak                            *
  *   electricityispower@gmail.com                                          *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -48,10 +48,13 @@ class Outputs : public Screen< Menu<MPD::Output> >
 		
 		virtual List *GetList() { return w; }
 		
+		virtual bool isMergable() { return true; }
+		
 		void FetchList();
 		
 	protected:
 		virtual void Init();
+		virtual bool isLockable() { return true; }
 };
 
 extern Outputs *myOutputs;
