@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2008-2010 by Andrzej Rybczak                            *
+ *   Copyright (C) 2008-2012 by Andrzej Rybczak                            *
  *   electricityispower@gmail.com                                          *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -54,37 +54,36 @@ class Visualizer : public Screen<Window>
 		
 		virtual bool allowsSelection() { return false; }
 		
+		virtual bool isMergable() { return true; }
+		
 		void SetFD();
 		void ResetFD();
 		void FindOutputID();
 		
-		static int WindowTimeout;
+		static const int WindowTimeout;
 		
 	protected:
 		virtual void Init();
+		virtual bool isLockable() { return true; }
 		
 	private:
-		void DrawSoundWave(int16_t *, ssize_t);
+		void DrawSoundWave(int16_t *, ssize_t, size_t, size_t);
 #		ifdef HAVE_FFTW3_H
-		void DrawFrequencySpectrum(int16_t *, ssize_t);
+		void DrawFrequencySpectrum(int16_t *, ssize_t, size_t, size_t);
 #		endif // HAVE_FFTW3_H
 		
 		int itsOutputID;
 		timeval itsTimer;
 		
 		int itsFifo;
+		unsigned itsSamples;
 #		ifdef HAVE_FFTW3_H
+		unsigned itsFFTResults;
 		unsigned *itsFreqsMagnitude;
 		double *itsInput;
 		fftw_complex *itsOutput;
 		fftw_plan itsPlan;
 #		endif // HAVE_FFTW3_H
-		
-		static const unsigned Samples;
-#		ifdef HAVE_FFTW3_H
-		static const unsigned FFTResults;
-#		endif // HAVE_FFTW3_H
-		
 };
 
 extern Visualizer *myVisualizer;
